"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JUnitFormatTransformer = void 0;
const utils_1 = require("../utils");
const node_stream_1 = require("node:stream");
const narrowing_1 = require("../narrowing");
// cli currently has spaces in multiples of four for junit format
const tab = '    ';
const timeProperties = [
    'testExecutionTimeInMs',
    'testTotalTimeInMs',
    'commandTimeInMs'
];
// properties not in cli junit spec
const skippedProperties = ['skipRate', 'totalLines', 'linesCovered'];
class JUnitFormatTransformer extends node_stream_1.Readable {
    testResult;
    constructor(testResult, options) {
        super(options);
        this.testResult = testResult;
        this.testResult = testResult;
    }
    _read() {
        this.format();
        this.push(null); // Signal the end of the stream
    }
    format() {
        const { summary } = this.testResult;
        this.push(`<?xml version="1.0" encoding="UTF-8"?>\n`);
        this.push(`<testsuites>\n`);
        this.push(`${tab}<testsuite name="force.apex" `);
        this.push(`timestamp="${summary.testStartTime}" `);
        this.push(`hostname="${summary.hostname}" `);
        this.push(`tests="${summary.testsRan}" `);
        this.push(`failures="${summary.failing}"  `);
        this.push(`errors="0"  `);
        this.push(`time="${(0, utils_1.msToSecond)(summary.testExecutionTimeInMs)}">\n`);
        this.buildProperties();
        this.buildTestCases();
        this.push(`${tab}</testsuite>\n`);
        this.push(`</testsuites>\n`);
    }
    buildProperties() {
        this.push(`${tab}${tab}<properties>\n`);
        Object.entries(this.testResult.summary).forEach(([key, value]) => {
            if ((0, narrowing_1.isEmpty)(value) || skippedProperties.includes(key)) {
                return;
            }
            if (timeProperties.includes(key)) {
                value = `${(0, utils_1.msToSecond)(value)} s`;
                key = key.replace('InMs', '');
            }
            if (key === 'outcome' && value === 'Passed') {
                value = 'Successful';
            }
            if (key === 'testStartTime') {
                value = (0, utils_1.formatStartTime)(value);
            }
            this.push(`${tab}${tab}${tab}<property name="${key}" value="${value}"/>\n`);
        });
        this.push(`${tab}${tab}</properties>\n`);
    }
    buildTestCases() {
        const testCases = this.testResult.tests;
        for (const testCase of testCases) {
            const methodName = JUnitFormatTransformer.xmlEscape(testCase.methodName);
            this.push(`${tab}${tab}<testcase name="${methodName}" classname="${testCase.apexClass.fullName}" time="${(0, utils_1.msToSecond)(testCase.runTime)}">\n`);
            if (testCase.outcome === "Fail" /* ApexTestResultOutcome.Fail */ ||
                testCase.outcome === "CompileFail" /* ApexTestResultOutcome.CompileFail */) {
                let message = (0, narrowing_1.isEmpty)(testCase.message) ? '' : testCase.message;
                message = JUnitFormatTransformer.xmlEscape(message);
                this.push(`${tab}${tab}${tab}<failure message="${message}">`);
                if (testCase.stackTrace) {
                    this.push(`<![CDATA[${testCase.stackTrace}]]>`);
                }
                this.push(`</failure>\n`);
            }
            this.push(`${tab}${tab}</testcase>\n`);
        }
    }
    static xmlEscape(value) {
        return value
            .replace(/&/g, '&amp;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&apos;');
    }
}
exports.JUnitFormatTransformer = JUnitFormatTransformer;
__decorate([
    (0, utils_1.elapsedTime)()
], JUnitFormatTransformer.prototype, "format", null);
__decorate([
    (0, utils_1.elapsedTime)()
], JUnitFormatTransformer.prototype, "buildProperties", null);
__decorate([
    (0, utils_1.elapsedTime)()
], JUnitFormatTransformer.prototype, "buildTestCases", null);
//# sourceMappingURL=junitFormatTransformer.js.map